package ru.dbotthepony.mc.otm.client.render.blockentity

import com.mojang.blaze3d.platform.GlStateManager
import com.mojang.blaze3d.platform.GlStateManager.DestFactor
import com.mojang.blaze3d.platform.GlStateManager.SourceFactor
import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import com.mojang.math.Vector3f
import net.minecraft.client.renderer.MultiBufferSource
import net.minecraft.client.renderer.block.model.ItemTransforms
import net.minecraft.client.renderer.blockentity.BlockEntityRenderer
import net.minecraft.client.renderer.blockentity.BlockEntityRendererProvider
import org.lwjgl.opengl.GL14
import org.lwjgl.opengl.GL14.GL_ADD
import org.lwjgl.opengl.GL14.GL_FUNC_ADD
import org.lwjgl.opengl.GL14.GL_MIN
import org.lwjgl.opengl.GL14.glBlendColor
import ru.dbotthepony.mc.otm.block.entity.matter.MatterReplicatorBlockEntity
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.client.render.lockBlendFunc
import ru.dbotthepony.mc.otm.client.render.popScissorRect
import ru.dbotthepony.mc.otm.client.render.pushScissorRect
import ru.dbotthepony.mc.otm.client.screen.widget.PowerGaugePanel
import ru.dbotthepony.mc.otm.core.normalizeAngleDeg

class MatterReplicatorRenderer(private val context: BlockEntityRendererProvider.Context) : BlockEntityRenderer<MatterReplicatorBlockEntity> {
	override fun render(
		tile: MatterReplicatorBlockEntity,
		partialTick: Float,
		pose: PoseStack,
		bufferSource: MultiBufferSource,
		packedLight: Int,
		packedOverlay: Int
	) {
		val item = tile.visualItemStack

		if (item.isEmpty) {
			return
		}

		pose.pushPose()

		pose.translate(0.5, 0.3, 0.5)
		pose.scale(0.3f, 0.3f, 0.3f)

		val diff = System.nanoTime() - tile.lastRender
		tile.lastRender = System.nanoTime()

		tile.renderRotation = normalizeAngleDeg(tile.renderRotation + diff / 10_000_000f)
		pose.mulPose(Vector3f.YP.rotationDegrees(tile.renderRotation))

		val model = context.itemRenderer.getModel(
			item,
			tile.level,
			null,
			0
		)

		val source = minecraft.renderBuffers().bufferSource()

		source.endBatch()

		context.itemRenderer.render(
			item,
			ItemTransforms.TransformType.NONE,
			false,
			pose,
			source,
			packedLight,
			packedOverlay,
			model
		)

		glBlendColor(1f, 1f, 1f, 1f - tile.visualProgress)

		RenderSystem.enableBlend()
		RenderSystem.blendFuncSeparate(
			SourceFactor.ONE_MINUS_CONSTANT_ALPHA,
			DestFactor.CONSTANT_ALPHA,
			SourceFactor.ZERO,
			DestFactor.ONE
		)

		lockBlendFunc = true

		try {
			source.endBatch()
		} finally {
			lockBlendFunc = false
		}

		RenderSystem.defaultBlendFunc()
		glBlendColor(0f, 0f, 0f, 0f)

		pose.popPose()
	}
}
